# Generated by Django 3.0.11 on 2021-06-30 14:17

from django.db import migrations, models
import django.db.models.deletion
import qrtoolkit_core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0006_auto_20210614_1338'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='apicall',
            name='code',
        ),
        migrations.AddField(
            model_name='apicall',
            name='link_url',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='api_calls', to='qrtoolkit_core.LinkUrl', validators=[qrtoolkit_core.validators.validate_is_api_mode_qrcode]),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='qrcode',
            name='mode',
            field=models.CharField(choices=[('kiosk', 'Kiosk Mode'), ('redirect', 'Redirect Mode'), ('info_page', 'Information Page Mode'), ('api_call', 'Api call Mode')], default='redirect', help_text="\nSets the mode this code is in.<br/>\nKiosk Mode: Show buttons to choose a link from<br/>\nRedirect Mode: Instantly redirects to the url with the highest priority.<br/>\nInformation Page Mode: Show basic info with links to different urls.<br/>\nApi call Mode: Will look for ApiCall's in the database that correspond to the selected link_url, and execute them. (Extended Kiosk mode)\n", max_length=16),
        ),
    ]



# Generated by Django 3.0.11 on 2021-06-14 13:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0005_remove_erronous_hits'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApiCall',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.URLField()),
                ('payload', models.TextField()),
            ],
        ),
        migrations.AlterField(
            model_name='qrcode',
            name='mode',
            field=models.CharField(choices=[('kiosk', 'Kiosk Mode'), ('redirect', 'Redirect Mode'), ('info_page', 'Information Page Mode')], default='redirect', help_text='\nSets the mode this code is in.<br/>\nKiosk Mode: Show buttons to choose a link from<br/>\nRedirect Mode: Instantly redirects to the url with the highest priority.<br/>\nInformation Page Mode: Show basic info with links to different urls.\n', max_length=16),
        ),
        migrations.CreateModel(
            name='Header',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=128)),
                ('value', models.CharField(max_length=128)),
                ('api_call', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='headers', to='qrtoolkit_core.ApiCall')),
            ],
        ),
        migrations.AddField(
            model_name='apicall',
            name='code',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='api_calls', to='qrtoolkit_core.QRCode'),
        ),
    ]



# Generated by Django 3.0.11 on 2021-09-28 11:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qrtoolkit_core', '0008_auto_20210630_1526'),
    ]

    operations = [
        migrations.AddField(
            model_name='linkurl',
            name='visible',
            field=models.BooleanField(default=True, help_text='Sets the visibility for this link in KIOSK mode.'),
        ),
    ]



from django.urls import reverse
from rest_framework.renderers import TemplateHTMLRenderer, JSONRenderer
from rest_framework.response import Response
from rest_framework.exceptions import NotAuthenticated
from .models import ApiHit, QRCode, LinkUrl
from django.http import Http404, HttpResponse
from django.conf import settings
from rest_framework.views import APIView
from django.shortcuts import redirect
from django.core.exceptions import ValidationError
from django.views.generic import DetailView
from django.shortcuts import get_object_or_404
import uuid
import logging

logger = logging.getLogger('qrtoolkit_core.views')


class CodeView(DetailView):
    template_name = 'qrtoolkit_core/qrcode/code.html'
    pk_url_kwarg = 'short_uuid'
    queryset = QRCode.objects.all()
    context_object_name = 'code'

    def get_object(self, queryset=None):
        uid = self.kwargs.get(self.pk_url_kwarg)
        try:
            try:
                short_uuid = uuid.UUID(uid)
                # short_uuid is a valid uuid object
                return self.queryset.get(uuid=short_uuid)
            except ValueError:
                # short_uuid is not a valid uuid object
                return self.queryset.get(short_uuid=uid)
        except QRCode.DoesNotExist or ValidationError:
            raise Http404


class QRCodeDetails(APIView):
    renderer_classes = [TemplateHTMLRenderer, JSONRenderer]
    permission_classes = []

    @staticmethod
    def get_object(short_uuid):
        try:
            try:
                short_uuid = uuid.UUID(short_uuid)
                # short_uuid is a valid uuid object
                return QRCode.objects.get(uuid=short_uuid)
            except ValueError:
                # short_uuid is not a valid uuid object
                return QRCode.objects.get(short_uuid=short_uuid)

        except (QRCode.DoesNotExist or ValidationError) as error:
            hit = ApiHit(code=None, action=ApiHit.ACTION_CHOICES.ERROR,
                         message=f'code with id {{{str(short_uuid)}}} does not exist.')
            hit.save()
            raise Http404

    def get(self, request, short_uuid=None, format=None):
        qrcode = self.get_object(short_uuid=short_uuid)

        if request.accepted_renderer.format == 'json' or format == 'json':
            hit = ApiHit(
                code=qrcode, action=ApiHit.ACTION_CHOICES.JSON)
            hit.save()
            response = redirect(to=f'{settings.API_URL}/qrcodes/{qrcode.id}/', permanent=False)
            auth_header = request.headers.get('apiKey')
            if auth_header is None:
                raise NotAuthenticated
            response['apiKey'] = auth_header
            return response

        if request.accepted_renderer.format == 'html' or format == 'html':
            if qrcode.urls.count() == 0:
                hit = ApiHit(code=qrcode, action=ApiHit.ACTION_CHOICES.BASIC_INFO)
                hit.save()
                return Response({'qrcode': qrcode}, template_name='qrtoolkit_core/qrcode/index.html')

            if qrcode.mode == QRCode.REDIRECT_MODE_CHOICES.REDIRECT:
                hit = ApiHit(
                    code=qrcode, action=ApiHit.ACTION_CHOICES.REDIRECT)
                hit.save()
                return redirect(qrcode.urls.first().url, permanent=False)
            if qrcode.mode == QRCode.REDIRECT_MODE_CHOICES.KIOSK:
                hit = ApiHit(code=qrcode, action=ApiHit.ACTION_CHOICES.KIOSK)
                hit.save()
                return Response({'qrcode': qrcode}, template_name='qrtoolkit_core/qrcode/kiosk.html')
            if qrcode.mode == QRCode.REDIRECT_MODE_CHOICES.INFO_PAGE:
                hit = ApiHit(code=qrcode, action=ApiHit.ACTION_CHOICES.BASIC_INFO)
                hit.save()
                return Response({'qrcode': qrcode}, template_name='qrtoolkit_core/qrcode/index.html')
            if qrcode.mode == QRCode.REDIRECT_MODE_CHOICES.API_CALL:
                hit = ApiHit(code=qrcode, action=ApiHit.ACTION_CHOICES.API_CALL)
                hit.save()
                return Response({'qrcode': qrcode}, template_name='qrtoolkit_core/qrcode/kiosk_call.html')

            return Response({'qrcode': qrcode}, template_name='qrtoolkit_core/qrcode/index.html')

        hit = ApiHit(
            code=qrcode, action=ApiHit.ACTION_CHOICES.JSON)
        hit.save()
        return redirect(to=f'{settings.API_URL}/qrcodes/{qrcode.id}/', permanent=False)


def link_url_clicked(request, pk):
    u = get_object_or_404(LinkUrl, pk=pk)
    handle_api_calls(u)
    return redirect(to=u.url, permanent=False)


def handle_api_calls(url):
    """
    If the code has apicalls configured, fire them off.
    :type url: LinkUrl
    """
    import requests
    for api_call in url.api_calls.all():
        headers = {}
        for header in api_call.headers.all():
            headers[header.key] = header.value
        logger.info(f'posting api_call from {url.name} to {api_call.url}')
        r = requests.post(api_call.url, api_call.payload, headers=headers)
        logger.info(f'response: {r.content.decode("utf-8")}, status_code: {r.status_code}')
